@@ -1,7 +1,7 @@
 /*
  * FILE:     net_udp.c
  * AUTHOR:   Colin Perkins 
- * MODIFIED: Orion Hodson & Piers O'Hanlon
+ * MODIFIED: Orion Hodson, Piers O'Hanlon, Kristian Hasler
  * 
  * Copyright (c) 1998-2000 University College London
  * All rights reserved.
@@ -59,6 +59,10 @@
 const struct	in6_addr	in6addr_any = {IN6ADDR_ANY_INIT};
 #endif
 
+#ifdef WINXP_IPV6
+const struct	in6_addr	in6addr_any = {IN6ADDR_ANY_INIT};
+#endif
+
 /* This is pretty nasty but it's the simplest way to get round */
 /* the Detexis bug that means their MUSICA IPv6 stack uses     */
 /* IPPROTO_IP instead of IPPROTO_IPV6 in setsockopt calls      */
@@ -396,9 +400,11 @@ udp_sendv4(socket_udp *s, struct iovec *vector, int count)
 	msg.msg_namelen    = sizeof(s_in);
 	msg.msg_iov        = vector;
 	msg.msg_iovlen     = count;
+#ifdef NDEF	/* Solaris does something different here... can we just ignore these fields? [csp] */
 	msg.msg_control    = 0;
 	msg.msg_controllen = 0;
 	msg.msg_flags      = 0;
+#endif
 	return sendmsg(s->fd, &msg, 0);
 }
 #endif
@@ -511,7 +517,7 @@ static socket_udp *udp_init6(const char *addr, const char *iface, uint16_t rx_po
 		imr.ipv6mr_interface = 0;
 #endif
 		
-		if (SETSOCKOPT(s->fd, IPPROTO_IPV6, IPV6_ADD_MEMBERSHIP, (char *) &imr, sizeof(struct ipv6_mreq)) != 0) {
+		if (SETSOCKOPT(s->fd, IPPROTO_IPV6, IPV6_ADD_MEMBERSHIP, (char *) &imr, sizeof(imr)) != 0) {
 			socket_error("setsockopt IPV6_ADD_MEMBERSHIP");
 			return NULL;
 		}
@@ -614,9 +620,11 @@ udp_sendv6(socket_udp *s, struct iovec *vector, int count)
 	msg.msg_namelen    = sizeof(s_in);
 	msg.msg_iov        = vector;
 	msg.msg_iovlen     = count;
+#ifdef HAVE_MSGHDR_MSGCTRL  
 	msg.msg_control    = 0;
 	msg.msg_controllen = 0;
 	msg.msg_flags      = 0;
+#endif
 	return sendmsg(s->fd, &msg, 0);
 #else
 	UNUSED(s);
